function [cf,cf1,cf2,cf3]=cfmu(u,mu,cf0,cf01,cf02,cf03)
%CFMU	Combine mu value with characteristic function and derivatives
%
%	[cf,cf1,cf2,cf3]=cfmu(u,mu,cf0,cf01,cf02,cf03)
%
%	Output arguments:
%	cf = values of the characteristic function at u
%	cf1 = first derivative of the characteristic function at u
%	cf2 = second derivative of the characteristic function at u
%	cf3 = third derivative of the characteristic function at u
%
%	Input arguments:
%	u = values of the input argument (may be a vector)
%	mu = mean value (default value: zero)
%	cf0 = values of the characteristic function at u
%	cf01 = first derivative of the characteristic function at u
%	cf02 = second derivative of the characteristic function at u
%	cf03 = third derivative of the characteristic function at u
%	    cf03, (cf03,cf02) or (cf03,cf02,cf01) may be empty; in that case
%	    the corresponding output arguments will be empty, too.

%       Copyright (c) I. Kollar, 1994
%       All rights reserved.
%       Last modified: 01-Dec-1994

if nargin<6, cf03=[]; end
if nargin<5, cf02=[]; end
if nargin<4, cf01=[]; end
if length(mu)>1, error('mu is not a scalar'), end
if imag(mu)~=0, error('mu is not real'), end
if any(any(imag(u)~=0)), error('u is not real'), end
%
if any(size(u)~=size(cf0)), error('Sizes  of u and cf0 differ'), end
expum=exp(+j*u*mu);
cf=expum.*cf0;
cf1=[];
if (nargout>=2)&~isempty(cf01)
  if any(size(u)~=size(cf01)), error('Sizes  of u and cf01 differ'), end
  cf1=expum.*cf01...
	+(j*mu)*expum.*cf0;
end
%
cf2=[];
if (nargout>=3)&~isempty(cf02)
  if any(size(u)~=size(cf02)), error('Sizes  of u and cf02 differ'), end
  cf2=expum.*cf02...
	+2*(j*mu)*expum.*cf01...
	+(j*mu)^2*expum.*cf0;
end
%
cf3=[];
if (nargout>=4)&~isempty(cf03)
  if any(size(u)~=size(cf03)), error('Sizes  of u and cf03 differ'), end
  cf3=expum.*cf03...
	+3*(j*mu)*expum.*cf02...
	+3*(j*mu)^2*expum.*cf01...
        +(j*mu)^3*expum.*cf0;
end
%
%End of cfmu
