function [ylat,sk]=latcfilt_ap_q(k,x,Q,imp)
%LATCFILT_AP_Q  Lattice allpass filter, quantized
%
%imp: 1:3: f(imp)+1, 4:6, w(imp-3)+1

Nx=length(x);
sk=zeros(Nx,13);
ylat=zeros(size(x));
w=zeros(3,1); %Present value of memories: w(3), w(2), w(1)
if nargin<2, error('x is not given'), end
if nargin<3, Q=''; end
if isempty(Q), i1=1; else i1=max(1,lsb(Q)); end
if nargin<4, imp=[]; end
if ~isempty(imp)&~ismember(imp,[1:6]), error('imp invalid'), end
if ismember(imp,[4:6]), w(imp-3)=i1; end
for n=1:Nx
  f=roundq(x(n)-roundq(k(3)*w(3),Q),Q); %active summer output
  if (n==1)&(imp==3), f=f+i1; end
  sk(n,13)=f; %summer(1,1) output 
  sk(n,12)=roundq(-k(3)*w(3),Q); %branch -k(3)
  sk(n,11)=roundq(f*k(3),Q); %branch k(3)
  sk(n,10)=roundq(sk(n,11)+w(3),Q); %summer (2,1) output
  ylat(n)=sk(n,10); %output
  
  f=roundq(f-roundq(k(2)*w(2),Q),Q); %active summer output
  if (n==1)&(imp==2), f=f+i1; end
  sk(n,9)=f; %summer(1,2) output  
  sk(n,8)=roundq(-k(2)*w(2),Q); %branch -k(2)
  sk(n,7)=roundq(f*k(2),Q); %branch k(2)
  sk(n,6)=roundq(sk(n,7)+w(2),Q); %summer (2,2) output
  w(3)=sk(n,6); %stage 2 output

  f=roundq(f-roundq(k(1)*w(1),Q),Q); %active summer output
  if (n==1)&(imp==1), f=f+i1; end
  sk(n,5)=f; %summer(1,3) output
  sk(n,4)=roundq(-k(1)*w(1),Q); %branch -k(1)
  sk(n,3)=roundq(f*k(1),Q); %branch k(1)
  sk(n,2)=roundq(sk(n,3)+w(1),Q); %summer (2,3) output
  w(2)=sk(n,2); %stage 3 output 

  sk(n,1)=f;
  w(1)=f; %feedback
end
