function value=nextafter(x,y,p)
%NEXTAFTER  The next representable neighbor of x in the direction toward y
%       If x=y, the result is x; otherwise, in either of x and y is NaN, value=NaN.
%
%       value=nextafter(x,y,p)
%
%   In the place of p, structure of run modifiers can also be given, as
%       struct('p',p,'UnderFlow','flush to zero')
%
%   Examples: 
%       nextafter(1,2)-1  %is eps = 2^(-52) = 2.2204e-016
%       nextafter(1,0)-1  %is -eps/2 = -2^(-52-1) = -1.1104e-016
%       nextafter(1,2,24)-1  %is eps = 2^(-23) = 1.1921e-007
  
%	Copyright (c) I. Kollar, 2005, all rights reserved
%	Last modified: 4-Jan-2006

if ~isnumeric(x), error('x is not numeric'), end
if length(x)==0, value=[]; return, end
value=zeros(size(x));
%
lb='h'; Emin=-1022; Emax=1023; bias=[];
if ~exist('p'), p=''; end, if isempty(p), p=53; end
if ~exist('ufl'), ufl=''; end, if isempty(ufl), ufl='g'; end
if nargin>2
  runmod=p;
  if isstruct(runmod)
    p=53;
    fn=fieldnames(runmod);
    for ii=1:length(fn)
      if strcmp(fn{ii},'p'), p=getfield(runmod,'p');
      elseif strcmp(fn{ii},'Emin'), Emin=getfield(runmod,'Emin'); Emingiven=1;
      elseif strcmp(fn{ii},'Emax'), Emax=getfield(runmod,fn{ii});
      elseif strcmp(fn{ii},'bias'), bias=getfield(runmod,'Ebias');
      elseif strcmpi(fn{ii},'lb')|strncmpi(fn{ii},'leadingbit',1), 
        lb=getfield(runmod,fn{ii}); lb=lower(lb(1));
      elseif strcmpi(fn{ii},'underflow')|strncmpi(fn{ii},'ufl',2), 
        ufl=getfield(runmod,fn{ii}); ufl=lower(ufl(1));
      else error(['Unknown field ''',fn{ii},''' of runmod'])
      end
    end
  elseif isnumeric(runmod)
    p=runmod;
  end
end

if isstr(p)
  if strncmp(p,'single',1)
    lb='h'; p=24; Emin=-126; Emax=127; ufl='g';
  else 
    error(['p = ''',p,''' is not allowed'])
  end
else
  if isempty('Emin'), Emin=-1022; Emax=1023; end
end
%
for ii=1:prod(size(x))
  xi=x(ii);
  if isnan(xi)
    value(ii)=NaN; break
  else
    if ~isfinite(xi)
      if ~isequal(y,xi), value(ii)=sign(xi)*pow2(1-2^(-p+1),Emax+1); 
      else value(ii)=xi;
      end
      break
    else
      xp=qfloat(xi,qstruct('p',p,'ufl',ufl,'Emin',Emin,'Emax',Emax));
      if ~isequal(xi,xp)
        error(sprintf('Element %.0f of x has not precision p',ii))
      end
      if 1|isequal(y,xi) %only for representable numbers
        value(ii)=xi;
      elseif y<xi
        value(ii)=qfloat(xi,qstruct('p',p,'ufl',ufl,'Emin',Emin,'Emax',Emax,'rdir','floor'));
      elseif y>xi
        value(ii)=qfloat(xi,qstruct('p',p,'ufl',ufl,'Emin',Emin,'Emax',Emax,'rdir','ceil'));
      end
      if isequal(value(ii),xi)
        if isequal(xi,0)
          if strncmpi(ufl,'g',1), value(ii)=sign(y)*lsb(0);
          elseif strncmpi(ufl,'f',1), value(ii)=sign(y)*pow2(-2^(-p+1),Emin+1);;
          end
        else
          addv=lsb(xi,p);
          if isequal(rem(log2(abs(xi)),1),0)&(abs(y)<=abs(xi))
            value(ii)=xi-sign(xi)*addv/2;
          else
            value(ii)=xi+sign(xi)*addv;
          end
        end
      end
    end
    if strcmp(ufl,'f')
    elseif strcmp(ufl,'g')
    elseif isempty(ufl)
    else error('ufl is not allowed')
    end
  end
end %for ii
%
%End of nextafter