function [cf,cf1,cf2,cf3]=cfbinary(u,A,mu)
%CFBINARY Characteristic function of a binarily distributed random variable.
%
%	[cf,cf1,cf2,cf3]=cfbinary(u,A,mu)
%
%       Definitions:
%       f(x) = 0.5*delta(x-A) + 0.5*delta(x+A)
%       F(u) = cos(A*u)
%
%	Output arguments:
%       cf = values of the characteristic function at u
%       cf1 = first derivative of the characteristic function at u
%       cf2 = second derivative of the characteristic function at u
%       cf3 = third derivative of the characteristic function at u
%
%	Input arguments:
%	u = values of the input argument (may be a vector or an array)
%	A = amplitude (default value: 1)
%	mu = mean value (default value: zero)
%
%	See also: CFCALC

%       Copyright (c) I. Kollar, 1994
%       All rights reserved.
%       Last modified: 02-Aug-1994

if nargin<3, mu=[]; end, if isempty(mu), mu=0; end
if length(mu)>1, error('mu is not a scalar'), end
if imag(mu)~=0, error('mu is not real'), end
if nargin<2, A=[]; end, if isempty(A), A=1; end
if length(A)>1, error('A is not a scalar'), end
if imag(A)~=0, error('A is not real'), end
if A<=0, error('A is not positive'), end
if any(any(imag(u)~=0)), error('u is not real'), end
%
cf=cos(u*A);
%
cf1=[]; cf2=[]; cf3=[];
if nargout>=2
  cf1=A*(-sin(u*A));
end
if nargout>=3
  cf2=A^2*(-cos(u*A));
end
if nargout>=4
  cf3=A^3*(sin(u*A));
end
%
%Combine with mu if it differs from zero
if mu~=0
  [cf,cf1,cf2,cf3]=cfmu(u,mu,cf,cf1,cf2,cf3);
end %mu~=0
%
%End of cfbinary
