function [cf,cf1,cf2,cf3]=cftriangular(u,A,mu)
%CFTRIANGULAR Characteristic function of a triangularly distributed random variable.
%
%	cf=cftriangular(u,A,mu)
%
%	Definitions:
%	f(x) = 1/A*(1-abs(x)/A), abs(x)<A, A>0
%	F(u) = sinc(A*u)^2
%
%	Output arguments:
%       cf = values of the characteristic function at u
%       cf1 = first derivative of the characteristic function at u
%       cf2 = second derivative of the characteristic function at u
%       cf3 = third derivative of the characteristic function at u
%
%	Input arguments:
%	u = values of the input argument (may be a vector or an array)
%	A = amplitude (default value: 1)
%	mu = mean value (default value: zero)
%
%	See also: CFCALC, CFUNIFORM

%       Copyright (c) I. Kollar, 1994
%       All rights reserved.
%       Last modified: 22-Mar-2006

if nargin<3, mu=[]; end, if isempty(mu), mu=0; end
if length(mu)>1, error('mu is not a scalar'), end
if imag(mu)~=0, error('mu is not real'), end
if nargin<2, A=[]; end, if isempty(A), A=1; end
if length(A)>1, error('A is not a scalar'), end
if imag(A)~=0, error('A is not real'), end
if A<=0, error('A is not positive'), end
if any(any(imag(u)~=0)), error('u is not real'), end
%
cf1=[]; cf2=[]; cf3=[];
%cf is the square of the cf of the uniform distribution
if nargout>=4, [cf,cf1,cf2,cf3]=cfuniform(u,A,mu);
elseif nargout>=3, [cf,cf1,cf2]=cfuniform(u,A,mu);
elseif nargout>=2, [cf,cf1]=cfuniform(u,A,mu);
else cf=cfuniform(u,A,mu);
end
%
if nargout>=4
  cf3=2*cf.*cf3+6*cf1.*cf2;
end
if nargout>=3
  cf2=2*cf.*cf2+2*cf1.^2;
end
if nargout>=2
  cf1=2*cf.*cf1;
end
cf=cf.^2;
%
%Combine with mu if it differs from zero
if mu~=0
  [cf,cf1,cf2,cf3]=cfmu(u,mu,cf,cf1,cf2,cf3);
end %mu~=0
%
%End of cftriangular
