%FFT_COMPARISON_FIX with Matlab's fft, with and without dither, fixed-point
%
%    The main goal of this function is to compare the following two 
%    FFT algorithms using different quantizers (floating-point / fix-point, 
%    dither: on/off):
%
%    fft(), 
%    roundfft() using runmod.exponent = 'block-float' and
%                     runmod.algorithm = 'DIT'
%    
%   The input vector contains samples of a noiseless sinus function with
%   changing frequency.
  
clear;
close all;

Q16fix_d_off = qstruct('Name','Q16fix_d_off','B',16,'fb',14)
qstructget(Q16fix_d_off)

Q16fix_d_u1 = qstructset(Q16fix_d_off,'Name','Q16fix_d_u1','dither','on','tod','u1')
qstructget(Q16fix_d_u1)

% Q16fix_d_off_pqn = qstruct('Name','Q16fix_d_off_pqn','B',16,'fb',14,'Operation','PQN')
% qstructget(Q16fix_d_off_pqn)

Q32fix_d_off = qstruct('Name','Q32fix_d_off','B',32,'fb',30)
qstructget(Q32fix_d_off)

Q32fix_d_u1 = qstructset(Q32fix_d_off,'Name','Q32fix_d_u1','dither','on','tod','u1')
qstructget(Q32fix_d_u1)

% Q32fix_d_off_pqn = qstruct('Name','Q32fix_d_off_pqn','B',32,'fb',30,'Operation','PQN')
% qstructget(Q32fix_d_off_pqn)


M = 3:0.1:4.9;
N = 256;
t = 0:0.001:0.256-0.001;
fs = 1/0.001;
f = M./N*fs;
fperfs = f./fs;
fig=1;

jelek = [];
argf = 2*pi.*f;
for ff=1:length(f)
    for tt=1:length(t)
        argft(tt)=argf(ff)*t(tt);
    end
    [x]=sin(argft);
    jelek = [jelek ; x];
end

sizejelek=size(jelek);

% for sj1=1:sizejelek(1)
% figure(fig);
% fig = fig + 1;
% plot(t,jelek(sj1,:),'x');
% tit=['Az eredeti jel, f =  ',int2str(f(sj1))];
% title(tit);
% end

X_FFT=[];
 for sj1=1:sizejelek(1)
% figure(fig);
% fig = fig + 1;
 X=fft(jelek(sj1,:));
 X_FFT=[X_FFT; X];
% plot(abs(X_FFT(sj1,:)),'h');
% title(['A Matlab fft-je a(z) ' int2str(sj1) '. jelre']);
 end

%  Mindegyik jelre

algorithm = {'dit'};
operation = {'quantize' , 'PQN'};

 % Q = {Q16fix_d_off,Q16fix_d_u1,Q16fix_d_off_pqn,Q32fix_d_off,Q32fix_d_u1,Q32fix_d_off_pqn};
Q = {Q16fix_d_off,Q16fix_d_u1,Q32fix_d_off,Q32fix_d_u1};

% Csak a 16 bites kvantlkra:
% Q = {Q16_d_off,Q16_d_u1};

fid = fopen('fft_comparison_fix_results.txt','wt');

ossz_esetszam = sizejelek(1)*length(Q)*length(algorithm);
eset = 1;
for sj1=1:sizejelek(1)
for q = 1:length(Q)
   clear runmod;
   runmod.exponent = 'block-float';
   for k=1:length(algorithm)
      runmod.algorithm = algorithm(k);
      if (exist('X_roundfft','var')) clear X_roundfft; end
      x=jelek(sj1,:);
      X_roundfft=roundfft(x,Q{q},runmod);
      X_ROUNDFFT = (X_roundfft.data)*2^(X_roundfft.bfe);
%         figure(fig);
%         fig = fig + 1;
%         plot(abs(X_ROUNDFFT),'h'); 
%         title([int2str(sj1) '.jelalak, ' Q{q}.Name ' , ' algorithm{k} ' , ' runmod.exponent]);
      
      % X_DIFF = (Matlab fft-je)-(a roundfft fft-je), ez mg komplex:
      X_DIFF =  X_FFT(sj1,:)-X_ROUNDFFT;
      
      % ABS_X_DIFF = abs(X_DIFF), ez mr vals, ez a hiba rdekel minket
      ABS_X_DIFF = abs(X_DIFF);
      
      ABS_X_DIFF_ESET(eset,:) = abs(X_DIFF);
      
      ABS_X_ROUNDFFT = abs(X_ROUNDFFT);
      SUM_ABS_X_FFT = sum(abs(X_FFT(sj1,:)));
      SUM_ABS_X_ROUNDFFT = sum(abs(X_ROUNDFFT));
      MAX_ABS_X_FFT = max(abs(X_FFT(sj1,:)));
      MAX_ABS_X_ROUNDFFT = max(abs(X_ROUNDFFT));
%        figure(fig);
%        fig = fig + 1;
%        plot(ABS_X_DIFF,'p'); 
%        title(['Az eltrs a Matlab fft-jtl a(z) ' int2str(sj1) '.jelalak esetn, ' Q{q}.Name ' , ' algorithm{k} ' , ' runmod.exponent]);
      disp([int2str(sj1) '.jelalak' ' a(z) ' int2str(sizejelek(1)) ' kzl , kvantl: ' Q{q}.Name ' , algorithm: ' algorithm{k}]);
      disp([int2str(eset) '. eset a(z) ' int2str(ossz_esetszam) ' eset kzl...']);
      tortresz = rem(M(sj1),1);
      egeszresz = M(sj1)-tortresz;
      tort_string = num2str(tortresz);
      egesz_string = num2str(egeszresz);
      disp(['Saving ' Q{q}.Name '_' egesz_string 'M' tort_string(3:end) '_' int2str(N)  'N_' algorithm{k} '.mat' ' ...']);
      string = [Q{q}.Name '_' egesz_string 'M' tort_string(3:end) '_' int2str(N)  'N_' algorithm{k}];
      save([Q{q}.Name '_' egesz_string 'M' tort_string(3:end) '_' int2str(N)  'N_' algorithm{k} '.mat']);
      load_command = [Q{q}.Name '_' egesz_string 'M' tort_string(3:end) '_' int2str(N)  'N_' algorithm{k} '=load(''' Q{q}.Name '_' egesz_string 'M' tort_string(3:end) '_' int2str(N)  'N_' algorithm{k} '.mat'');'];
      evalin('base',load_command);
      
      fprintf(fid,[string  '  %2.0f' '.eset\n'],eset);
     % evalin('base',' type fft_comparison_fix_results.txt');
      save('fft_comparison_fix_results.mat');
      eset = eset+1;
   end
end
end

fclose(fid);
save('fft_comparison_fix_results.mat')
