function [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsbmag,varargin)
%SET_DITHER  Set the uses of dither

sdith=[];
probab=[];
hmtimes=[];
sumhmtimes=[];
if nargin<3
  error('Not enough input arguments in function set_dither.');
  return
else
  switch tod
    case 'userfunction'
      if ~isempty(varargin) dfun=varargin{1}; end
      if ~exist('dfun','var') error('Dither-function does not exist in function set_dither.'); return;
      else
        for i=1:length(dfun)
          sdith(i)=lsbmag*dfun(i);
          probab(i)=1/length(dfun);
        end

      end
    case 'u1'
      n=2^L;
      qd=lsbmag;
      sdith=[-2^(L-1):2^(L-1)-1]*lsbmag;
      for i=1:n
        probab(i)=1/n;
      end
    case 'u2'
      n=2^L;
      qd=lsbmag;
      sdith=([-2^(L-1):2^(L-1)-1]+.5)*lsbmag;
      for i=1:n
        probab(i)=1/n;
      end
    case 'u3'
      n=2^L;
      qd=lsbmag;
      sdith=[-2^(L-1):2^(L-1)]*lsbmag;
      probab(1)=1/(2*n);
      for i=2:n
        probab(i)=1/n;
      end
      probab(n+1)=probab(1);
    case 't1'
      n=2^L;
      qd=lsbmag;
      for i=1:n
        unif(i)=1/n;
      end
      sdith=[-2^(L)+1:2^(L)-1]*lsbmag;
      probab = conv(unif,unif);
    case 't2'
      n=2^L;
      qd=lsbmag;
      for i=1:n
        unif(i)=1/n;
      end
      probab1 = conv(unif,unif);
      probab = conv(probab1,[1/2 1/2]);
      sdith(1) = -2^L+0.5;
      for i=2:2*n
        sdith(i)=sdith(i-1)+qd;
      end
    case 't3'
      n=2^L;
      qd=lsbmag;
      for i=1:n
        unif(i)=1/n;
      end
      sdith(1) = -2^L*lsbmag;
      for i=2:2*n+1
        sdith(i)=sdith(i-1)+qd;
      end
      probab = conv(unif,unif);
      probab = [(qd^2/n^2)/(4*lsbmag^2)/2, probab, (qd^2/n^2)/(4*lsbmag^2)/2];
      probab((length(probab)-1)/2+1) = probab((length(probab)-1)/2+1) - (qd^2/n^2)/(4*lsbmag^2) ;
    otherwise
      % error('Unknown type of dither in function set_dither.')
      return
  end

  probab2=probab;
  for i=1:length(probab)
    while (mod(probab2(i),1) ~= 0)
      probab2 = probab2*10;
    end
  end
  k=1;
  for i=1:length(probab2)
    for j=1:length(probab2)
      gcd_probab2(k)=gcd(probab2(i),probab2(j));
      k=k+1;
    end
  end
  gcd_min=min(gcd_probab2);
  hmtimes=probab2/gcd_min;
  sumhmtimes=sum(hmtimes);
end
