function test_roundoff_calls(N)
%TEST_ROUNDOFF_CALLS  Sample use of the roundoff M-files

if nargin<1, N=[]; end, if isempty(N), N=100; end
%Examples in the book:

%ROUNDFLOAT
x=randn(10,1);
for p=[24,53]
  xq=roundfloat(x,p);
  dx=sqrt(mean((abs(x-xq)./abs(x)).^2)*12);
  if (p==53)&any(~isequal(x,xq)), error('Double precision rounding is not OK'), end
  if (p==24)&(dx<.2*2^(-24)), error('Single precision yields to small error'), end
  if (p==24)&(dx>3*2^(-24)), error('Single precision yields to large error'), end
end

%APPENDIX
Q = qstruct('p',24); %define a quantizer
a = roundq(1.1,Q); b = roundq(2.34,Q); %rounded input numbers
c = roundq(a*b,Q); %operation and subsequent roundoff
num2bin(c,Q) %show contents of binary number
%
Q_PQN=qstructset(Q,'Operation','PQN'); %replace quantization by PQN
c2 = roundq(a*b,Q_PQN); %operation and addition of PQN
num2bin(c2,Q_PQN) %show contents of binary number

%WEB
a = 1; b = 1+2^(-23);
one_plus_eps = roundq(a*b,'single') %basic usage: operation + quantization
two = roundq(1 + roundq(a*b,'single'),'single') %exactly 2
ulp(two,'single')  %ULP for 2: 1.1921e-007=2*2^(-24)=2^(-p+1)
lsb(two,'single')  %LSB for 2: 2.3842e-007=2*2^(-23)=2^(-p+1)
binstr = num2bin(one_plus_eps,'single')  %binary string
binplusdecstr = num2bin(one_plus_eps,'single','decimal')  %exponent shown as decimal
roundq(1+2^(-24),'s')-1  %single precision, returns 0
roundq(1+2^(-23),'s')-1  %single precision, returns 2^(-23)=1.1921e-007
roundq(realmin/2,struct('ufl','f'),'list')  %flush to zero, returns 0
roundq(realmin/2,struct('ufl','g'))  %gradual underflow
          %realmin/2=2^(-1022-1)=1.1125e-308
