function propval=qstructget(varargin)
%QSTRUCTGET  List properties of quantizer
%
%   Examples:
%     qstructget('d') %double precision
%     qstructget('single') %single precision
%     qstructget(qstruct('B',4,'fb',0))
%     qstructget('double','p') %precision of double floating-point

%	Copyright (c) I. Kollar, 1995-2006, all rights reserved
%	Last modified: 4-May-2006

error(nargchk(0,2,nargin))

if nargin==0
  qprops
  return
end

typ='';
if nargin<1, varargin={'double'}; end
Q=[];
if isstr(varargin{1})
  if strncmp(varargin{1},'single',1)|strncmp(varargin{1},'double',1)|...
      strcnmpi(varargin{1},'32-bit',3)|strncmpi(varargin{1},'Q32bit',3)|...
      strncmpi(varargin{1},'16-bit',3)|strncmpi(varargin{1},'Q16bit',3)|...
      strncmpi(varargin{1},'8-bit',2)|strncmpi(varargin{1},'Q8bit',2)
    Q=qstruct(varargin{1});
  else
    error(['runmod ''',runmod,''' is not allowed'])
  end
elseif isstruct(varargin{1})|(length(varargin)==1)
    %varargin{1}
    %if length(fieldnames(Q))>=13, 
    if ~isfield(varargin{1},'Dither')
      error('The quantizer has no ''Dither'' field')
    end
    if strcmpi(varargin{1}.Dither,'on')
        if ~isfield(varargin{1},'Other')
           error('The quantizer has no Dither ''Other'' field')
        end
    end
    Q=varargin{1};
    %else Q=qstruct(varargin{1});
    %end
else %old call
  typ='floating-point';
end
if ~isempty(Q) 
  fn=fieldnames(Q);
  if isstruct(Q)&(length(fn)<13), error('Not enough quantizer properties given'), end
  for ii=1:length(fn)
    if strcmp(fn{ii},'Type'), typ=Q.Type;
    elseif strcmp(fn{ii},'Phase'), ph=Q.Phase;
    elseif strcmpi(fn{ii},'Dither'), dith=Q.Dither; 
    else eval([fn{ii},'=getfield(Q,fn{ii});'])
      if strcmp(fn{ii},'Name'), name=Name;
      elseif strcmp(fn{ii},'Precision'), p=Precision;
      elseif strcmp(fn{ii},'Bits'), B=Bits;
      elseif strcmp(fn{ii},'FractionBits'), fb=FractionBits;
      elseif strcmp(fn{ii},'RoundingDir'), rdir=RoundingDir;
      elseif strcmp(fn{ii},'TreatHalf'), treath=TreatHalf;
      elseif strcmp(fn{ii},'Coding'), coding=Coding;
      elseif strcmp(fn{ii},'OverFlow'), ovf=OverFlow;
      elseif strcmp(fn{ii},'Minimum'), xnmin=Minimum;
      elseif strcmp(fn{ii},'Maximum'), xpmax=Maximum;
      elseif strcmp(fn{ii},'ExpMinimum'), Emin=ExpMinimum;
      elseif strcmp(fn{ii},'ExpMaximum'), Emax=ExpMaximum;
      elseif strcmp(fn{ii},'ExpBias'), Ebias=ExpBias;
      elseif strcmp(fn{ii},'Operation'), op=Operation;
      elseif strcmp(fn{ii},'LeadingBit'), lb=LeadingBit;
      elseif strcmp(fn{ii},'UnderFlow'), ufl=UnderFlow;
          
      %elseif strcmpi(fn{ii},'Dither'), dith=Dither; 
      elseif strcmpi(fn{ii},'ToD'), tod=ToD;
      elseif strcmpi(fn{ii},'Number_of_DitherBits'), L=Number_of_DitherBits;
      elseif strcmpi(fn{ii},'Amplitude_of_Sinusoid_Dither'), A=Amplitude_of_Sinusoid_Dither;  
      elseif strcmpi(fn{ii},'Ratio_of_Frequencies'), RoFn=Ratio_of_Frequencies(1);  RoFd=Ratio_of_Frequencies(2);
      %elseif strcmpi(fn{ii},'Phase'), ph=Phase;
      elseif strcmpi(fn{ii},'Variance_of_Gaussian_Dither'), gvar=Variance_of_Gaussian_Dither;
      elseif strcmpi(fn{ii},'DFunction'), dfun=Dfunction;
      elseif strcmpi(fn{ii},'Sequenceperm'), seqp=Sequenceperm;
      elseif strcmpi(fn{ii},'Other'), other=Other;    
      else error(['Unknown field ''',fn{ii},''''])
      end
    end
  end
  if ~exist('p')|isempty(p), p=B-~strncmp(coding,'u',1); end
end
%
if nargin==2
  prop=qprops(varargin{2});
  if nargout==0
    display(getfield(Q,prop));
  else
    propval=getfield(Q,prop);
  end
  if strcmp(Q.Type, 'floating-point')
    if ~isequal(Q.Precision,Q.FractionBits+1), error('FractionBits must be precision-1'), end
  end
  return
end
if strcmp(typ,'fixed-point')
  if ~exist('p')|isempty(p), p=B-~strncmp(coding,'u',1); end
  if length(varargin)==1
  else %numeric B or old call
    %	[xprime,ovfind]=qfix(x,B,fb,rdir,treath,coding,ovf,xnmin,xpmax)
    if ~isempty(varargin), B = varargin{1}; end
    if length(varargin)>=2, fb=varargin{2}; end
    if length(varargin)>=3, rdir=varargin{3}; end
    if length(varargin)>=4, treath=varargin{4}; end
    if length(varargin)>=5, coding=varargin{5}; end
    if length(varargin)>=6, ovf=varargin{6}; end
    if length(varargin)>=7, xnmin=varargin{7}; end
    if length(varargin)>=8, xpmax=varargin{8}; end
    if ~exist('p')|isempty(p), p=B-~strncmp(coding,'u',1); end
  end
  %
  if  ~isempty(typ)&~strncmpi(typ,'fixed-point',3)&~strncmpi(typ,'floating-point',3)
    error(['Type ''',typ,''' is not fixed-point or floating-point'])
  end
  if length(p)>1, error('p is not scalar'), end
  if p<=1, error('bit number is not positive'), end
  if rem(p,1)~=0, error('p is not integer'), end
  if ~isempty(B)&~isempty(p)&~isequal(B,p+~strncmp(coding,'u',1))
    error('Both p and B are given, inconsistently with coding')
  end
  if isempty(p), p=15; end
  if isempty(B), B=p+1; end
  %In old routine, p includes sign
  %p=p+1;
  %
  if isnan(fb), fb=[]; end
  if isempty(fb), fb=0; end %decimal point at right
  if fb<0, error('fb is negative'), end
  if fb>B-~strncmp(coding,'u',1), error('fb is larger than number of binary bits'), end
  if rem(fb,1)~=0, error('fb is not integer'), end
  if isnan(rdir), rdir=''; end
  if isempty(rdir), rdir='r'; end %rounding
  if isnan(treath), treath=''; end
  if isempty(treath), treath='e'; end %round to even
  if isnan(coding), coding=''; end
  if isempty(coding), coding='t'; end %two's complement
  if strncmp(coding,'two''s complement',1)|strncmp(coding,'one''s complement',1)|...
      strncmp(coding,'sign-magnitude',1)|strncmp(coding,'unsigned',1)
  else error(['coding=''',coding,''' is not allowed'])
  end
  if isnan(ovf), ovf=''; end
  if isempty(ovf), ovf='m'; end %modular
  if isnan(xnmin), xnmin=[]; end
  if strncmp(coding,'t',1), xnminl=-pow2(B-fb-1); 
  else xnminl=-pow2(pow2(B-1)-1,-fb);
  end
  if isempty(xnmin), xnmin=xnminl; end
  if xnmin<xnminl, error('xnmin is too small for the given bit number'), end
  if isnan(xpmax), xpmax=[]; end
  xpmaxl=pow2(pow2(B-1)-1,-fb);
  if isempty(xpmax), xpmax=xpmaxl; end
  if xpmax>xpmaxl, error('xpmax is too large for the given bit number'), end
  if xpmax<xnmin, error('xpmax < xnmin'), end
  %
  if strncmp(rdir,'t',1) %truncate
    if strncmp(coding,'s',1), rdir='x'; %fix for sign-magnitude
    else rdir='f'; %floor for the other cases
    end
  end
  pmax=-log2(eps)+1; %53 for ieee
  %
  %Make list
  fprintf('Number representation:\n')
  fprintf('  Type: fixed-point\n')
  %fprintf('  Bit number: %.0f\n',B)
  %fprintf('  Number of fractional bits (bits to the right from binary point): %.0f\n',fb)
  if strncmp(coding,'u',1)
    %fprintf('  String: %.0f bits + binary point + %.0f bits of an unsigned non-negative number\n',B-fb,fb)
    fprintf('  String: %.0f.%.0f bits  (%.0f integer + %.0f fractional = %.0f bits = B), unsigned non-negative number\n',B-fb,fb,B-fb,fb,B)
  else
    %fprintf('  String: sign + %.0f bits + binary point + %.0f bits\n',B-1-fb,fb)
    fprintf('  String: s%.0f.%.0f bits (1 sign + %.0f integer + %.0f fractional, B = %.0f bits)\n',B-1-fb,fb,B-1-fb,fb,B)
  end
  if strcmp(rdir,'r'), str='round';
  elseif strcmp(rdir,'f'), str='floor'; 
  elseif strcmp(rdir,'c'), str='ceil'; 
  elseif strcmp(rdir,'x'), str='fix'; 
  elseif strcmp(rdir,'i'), str='toinf'; 
  elseif strcmp(rdir,'t'), str='trunc';
  end
  fprintf(['  Direction of quantization: ',str,'\n'])
  %
  if strcmp(treath,'f'), str='floor'; 
  elseif strcmp(treath,'c'), str='ceil'; 
  elseif strcmp(treath,'x'), str='fix'; 
  elseif strcmp(treath,'i'), str='toinf'; 
  elseif strcmp(treath,'e'), str='toeven';
  elseif strcmp(treath,'o'), str='toodd';
  elseif strcmp(treath,'r'), str='rand';
  end  
  fprintf(['  Way of treating *.5 values for round: ',str,'\n'])
  %
  if strncmp(coding,'s',1), str='sign-magnitude'; 
  elseif strncmp(coding,'o',1), str='one''s complement'; 
  elseif strncmp(coding,'t',1), str='two''s complement'; 
  elseif strncmp(coding,'u',1), str='unsigned'; 
  else error('coding is not allowed')
  end
  fprintf(['  Coding: ',str,'\n'])
  %
  if strncmp(ovf,'c',1), str='clip at bounds'; 
  elseif strncmp(ovf,'m',1), str='modular'; 
  elseif strncmp(ovf,'t',1), str='triangular'; 
  elseif strncmp(ovf,'i',1), str='infinity at overflow'; 
  elseif strncmp(ovf,'u',1), str='none'; 
  else error('ovf is not allowed')
  end
  fprintf(['  Overflow handling: ',str,'\n'])
  shbitn=num2str(floor(B/log2(10)));
  shfbn=num2str(ceil(fb/log2(10)));
  fprintf(['  Bounds of numbers: [%',shbitn,'.',shfbn,'f,%',shbitn,'.',shfbn,'f]\n'],xnmin,xpmax)
  %xpmax+xnmin
else %floating-point
  if length(varargin)==1
    %done
  else %numeric p or old call
    if ~isempty(varargin), p = varargin{1}; end
    if length(varargin)>=2, lb=varargin{2}; end
    if length(varargin)>=3, rdir=varargin{3}; end
    if length(varargin)>=4, treath=varargin{4}; end
    if length(varargin)>=5, coding=varargin{5}; end
    if length(varargin)>=6, ufl=varargin{6}; end
    if length(varargin)>=7, Emin=varargin{7}; end
    if length(varargin)>=8, Emax=varargin{8}; end
  end
  %
  pmax=-log2(eps)+1; %53 for IEEE double precision
  if ~exist('p'), p=[]; end, if isempty(p), p=53; end
  if rem(p,1)~=0, error('p is not integer'), end
  fb=p-1;
  %
  if ~exist('lb'), lb=''; end, if isnan(lb), lb=''; end
  if isempty(lb), lb='h'; end %hidden leading bit
  if strncmp(lb,'h',1), hbn=1; elseif strncmp(lb,'s',1), hbn=0;
  else error(['lb=''',lb,''' is not allowed'])
  end
  if p>pmax, error(sprintf('bit number p is larger than %.0f',pmax)), end
  if p<2, error('bit number of significand is not positive'), end
  %
  if ~exist('rdir'), rdir=''; end, if isnan(rdir), rdir=''; end
  if isempty(rdir), rdir='r'; end %round
  if ~exist('treath'), treath=''; end, if isnan(treath), treath=''; end
  if isempty(treath), treath='e'; end %round to even
  if ~exist('coding'), coding=''; end, if isempty(coding), coding='s'; end %sign-magnitude
  if ~exist('ufl'), ufl=''; end, if isnan(ufl), ufl=''; end
  if isempty(ufl), ufl='g'; end %gradual underflow
  %
  Eminlim=round(log2(realmin)); Emaxlim=round(log2(realmax))-1;
  %For IEEE double precision, Emin=-1022, Emax=1023
  if ~exist('Emin'), Emin=[]; end, if isnan(Emin), Emin=[]; end
  if isempty(Emin), Emin=Eminlim; end
  if ~exist('Emax'), Emax=[]; end, if isnan(Emax), Emax=[]; end
  if isempty(Emax), Emax=Emaxlim; end
  if Emin<Eminlim
    error(sprintf('Emin=%.3g is too small for this computer, limit: %.0f',...
      Emin,Eminlim))
  end
  if Emax>Emaxlim
    error(sprintf('Emax=%.3g is too large for this computer, limit: %.0f',...
      Emax,Emaxlim))
  end
  if Emax<Emin, error('Emax is smaller than Emin'), end
  %
  %Make list
  fprintf('Number representation:\n')
  fprintf('  Name: %s\n',name)
  fprintf('  Type: floating-point\n')
  if isstr(varargin{1})
    if strncmp(varargin{1},'double',1), fprintf('  IEEE double precision\n')
    elseif strncmp(varargin{1},'single',1), fprintf('  IEEE single precision\n')
    end
  end
  fprintf('  Precision: %.0f\n',p)
  fprintf('  Number of bits: sign+(leading bit)+.[precision-1]+exponent = %.0f+%.0f+%.0f+%.0f = %.0f\n',...
    1,strcmp(lb,'s'),p-1,ceil(log2(Emax-Emin)),...
    1+strcmp(lb,'s')+p-1+ceil(log2(Emax-Emin)))
  if strcmp(lb,'h'), str='hidden'; else str='shown'; end
  fprintf(['  Leading bit representation: ',str,'\n'])
  if strcmp(rdir,'r'), str='round'; 
  elseif strcmp(rdir,'f'), str='floor'; 
  elseif strcmp(rdir,'c'), str='ceil'; 
  elseif strcmp(rdir,'x'), str='fix'; 
  elseif strcmp(rdir,'i'), str='toinf'; 
  elseif strcmp(rdir,'t'), str='trunc';
  end
  fprintf(['  Direction of quantization: ',str,'\n'])
  if strcmp(treath,'f'), str='floor'; 
  elseif strcmp(treath,'c'), str='ceil'; 
  elseif strcmp(treath,'x'), str='fix'; 
  elseif strcmp(treath,'i'), str='toinf'; 
  elseif strcmp(treath,'e'), str='toeven';
  elseif strcmp(treath,'o'), str='toodd';
  elseif strcmp(treath,'r'), str='rand';
  end  
  fprintf(['  Way of treating *.5 values for round: ',str,'\n'])
  if strcmp(coding,'s'), str='sign-magnitude'; 
  elseif strcmp(coding,'o'), str='one''s complement'; 
  elseif strcmp(coding,'t'), str='two''s complement'; 
  elseif strcmp(coding,'u'), str='unsigned'; 
  end
  fprintf(['  Coding of mantissa: ',str,'\n'])
  if strcmp(ufl,'g'), str='gradual'; 
  elseif strcmp(ufl,'f'), str='flush to zero'; 
  end
  fprintf(['  Underflow: ',str,'\n'])
  fprintf(['  Bounds of exponent: [%.0f,%.0f]\n'],Emin,Emax)  
  if exist('Ebias')
    if ~isempty(Ebias)&~isequal(Ebias,0)
      if Ebias<=-Emin
        error(sprintf('Ebias = %.0f is not larger than -Emin = %.0f',Ebias,-Emin))
      else
        fprintf(['  Bias in exponent representation: %.0f\n'],Ebias)
      end
    end
  end
end

if ~strcmpi(dith,'on')
    str='off';
    %fprintf(['  Using Dither: ',str,'\n']);
else str='on';
  fprintf('Dither properties:\n')
  fprintf(['  Using Dither: ',str,'\n'])
  if strncmpi(tod,'u1',2) str='Uniform 1'; end
  if strncmpi(tod,'u2',2) str='Uniform 2'; end
  if strncmpi(tod,'u2',2) str='Uniform 3'; end
  if strncmpi(tod,'t1',2) str='Triangular 1'; end
  if strncmpi(tod,'t2',2) str='Triangular 2'; end
  if strncmpi(tod,'t3',2) str='Triangular 3'; end
  if strncmpi(tod,'sin',3) str='Sinusoid'; end
  if strncmpi(tod,'gaussian',2) str='Gaussian'; end
  if strncmpi(tod,'user',2) str='Given by User'; end
  fprintf(['  Type of Dither: ',str,'\n'])
  fprintf(['  Number of DitherBits: %d \n'],L)
  switch str
    case 'Sinusoid',
      fprintf(['  Amplitude of Sinusoid Dither: %.0f \n'],A);
      fprintf(['  Phase of Sinusoid Dither: %.0f \n'],ph);
      fprintf(['  Ratio of Frequencies at Sinusoid Dither: %.0f/%.0f\n'],RoFn,RoFd);
      fprintf(['  Sequenceperm: ',seqp,'\n']);
    case 'Gaussian',
      fprintf(['  Variance of Gaussian Dither: %.0f\n'],gvar)
      fprintf(['  Sequenceperm: ',seqp,'\n']);
    case 'Given by User',
      fprintf(['  Elements of the following vector are used as dither:\n']);
      fprintf('      '), fprintf('%.4g   ',dfun); disp(' ')
      fprintf(['  Sequenceperm: ',seqp,'\n']);
  end

  [a,b,c,d,e] = get_dither_properties(Q);
  if ~isempty(a)
    fprintf(['  Other dither properties: \n']);
    fprintf(['    Dither values: \n']);
    fprintf('      '), fprintf('%.4g   ',[a]); disp(' ')
    fprintf(['    The probability of these dither values by order of succession: \n']);
    fprintf('      '), fprintf('%.4g   ',[b]); disp(' ')
    fprintf(['    Number of repetitions in a period: \n']);
    fprintf('      '), fprintf('%.4g   ',[c]); fprintf('Sum: %.0f\n',d)
    fprintf(['    The number of usage of these dither values by order of succession: \n']);
    fprintf('      '), fprintf('%.4g   ',[e]); disp(' ')
  end
end

if nargout==1, propval=Q; end
%
%End of qstructget