function Qout=qstructset(varargin)
%QSTRUCTSET Set properties of quantizer
%
%   Qout=qstruct(qdef,prop1,prop1val,prop2,prop2val)
%
%   Input arguments (optional):
%     qdef = structure of properties (fields) with values
%     propi,propival = property/value pairs
%     properties and values: see 'qstructget'
%
%   Defaults:
%     IEEE double precision: p=53, lb='hidden', rdir='round',
%         treath='toeven', coding='s', ufl='gradual', Emin=-1022, Emax=1023
%     IEEE single precision: p=24, lb='hidden', rdir='round',
%         treath='toeven', coding='s', ufl='gradual', Emin=-126, Emax=127
%   Examples:
%     Q=qstruct('double','prec',50) %IEEE double but 50-bit precision
%     Q2=qstructset(Q,'Precision',20) %change precision in Q to 20

%	Copyright (c) I. Kollar, 1995-2006, all rights reserved
%	Last modified: 4-May-2006

typ='';
fb=[]; lb=''; rdir=''; treath=''; coding=''; ufl=''; Emin=[]; Emax=[]; Ebias=0;
xnmin=[]; xpmax=[]; p=[]; B=[]; ovf=''; op='quantize'; Name='';dith=''; other=[];

if (nargin==0)|(nargin==1)
  if (nargin==1)&strcmpi(varargin{1},'dither'), qprops dither
  else qprops
  end
  return
end

if length(varargin)<1
  qdef='';
else
  qdef=varargin{1}; varargin(1)=[];
end
if ~isstruct(qdef), error('qdef is not a quantizer structure'), end
%
if isempty(qdef), qdef='double'; end
if isstr(qdef)
  if isempty(qdef)
  elseif strcmpi(qdef,'s')|strcmpi(qdef,'single')
    p=24; Emin=-126; Emax=127; Ebias=-Emin+1;
  elseif strcmpi(qdef,'d')|strcmpi(qdef,'double')|strcmpi(qdef,'list')
    p=53; Emin=-1022; Emax=1023; Ebias=-Emin+1;
  elseif strncmpi(qdef,'32',2)|strcmpi(qdef,'32-bit')
    B=32; p=B-1;
  elseif strncmpi(qdef,'16',2)|strcmpi(qdef,'16-bit')
    B=16; p=B-1;
  elseif strncmpi(qdef,'8',1)|strcmpi(qdef,'8-bit')
    B=8; p=B-1;
  elseif strcmpi(qdef,'list')
  else
    if (length(varargin)>0)&(rem(length(varargin),2)==1), varargin=[{qdef},varargin];
    else error(['qdef = ''',qdef,''' not allowed'])
    end
  end
elseif isstruct(qdef)
  fn=fieldnames(qdef);
  for ii=1:length(fn)
    %eval([fn{ii},'=getfield(qdef,fn{ii});'])
    if strcmp(fn{ii},'p')|strcmp(fn{ii},'prec')|strcmpi(fn{ii},'precision')
      p=getfield(qdef,fn{ii}); if isempty(typ), typ='floating-point'; end
    elseif strncmpi(fn{ii},'Type',2)
      typ=getfield(qdef,fn{ii});
    elseif strncmpi(fn{ii},'Name',3)
      Name=getfield(qdef,fn{ii});
    elseif strcmp(fn{ii},'lb')|strcmpi(fn{ii},'leadingbit')
      lb=getfield(qdef,fn{ii}); if isempty(typ), typ='floating-point'; end
    elseif strcmp(fn{ii},'rdir')|strncmpi(fn{ii},'roundingdirection',11)
      rdir=getfield(qdef,fn{ii});
    elseif strcmp(fn{ii},'treath')|strncmpi(fn{ii},'treathalf',6)
      treath=getfield(qdef,fn{ii});
    elseif strncmpi(fn{ii},'coding',1)
      coding=getfield(qdef,fn{ii});
    elseif strcmp(fn{ii},'ufl')|strcmpi(fn{ii},'underflow')
      ufl=getfield(qdef,fn{ii}); if isempty(typ), typ='floating-point'; end
    elseif strcmpi(fn{ii},'Emin')|strncmpi(fn{ii},'Expminimum',6)
      Emin=getfield(qdef,fn{ii}); if isempty(typ), typ='floating-point'; end
    elseif strcmpi(fn{ii},'Emax')|strncmpi(fn{ii},'Expmaximum',6)
      Emax=getfield(qdef,fn{ii}); if isempty(typ), typ='floating-point'; end
    elseif strcmpi(fn{ii},'Ebias')|strncmpi(fn{ii},'Expbias',6)
      Ebias=getfield(qdef,fn{ii}); if isempty(typ), typ='floating-point'; end
    elseif strcmp(fn{ii},'B')|strncmpi(fn{ii},'bits',4)
      B=getfield(qdef,fn{ii}); if isempty(typ), typ='fixed-point'; end
      if ~isempty(xnmin)&~isempty(fb)
        xnmin=-pow2((B-1)-fb); xpmax=pow2((B-1)-fb)-1;
        if strcmp(coding,'o')|strcmp(coding,'s'), xnmin=xnmin+1; end
      end
    elseif strcmpi(fn{ii},'xnmin')|strncmpi(fn{ii},'minimum',3)
      xnmin=getfield(qdef,fn{ii}); if isempty(typ), typ='fixed-point'; end
    elseif strcmpi(fn{ii},'xpmax')|strncmpi(fn{ii},'maximum',3)
      xpmax=getfield(qdef,fn{ii}); if isempty(typ), typ='fixed-point'; end
    elseif strcmpi(fn{ii},'ovf')|strncmpi(fn{ii},'overflow',5)
      ovf=getfield(qdef,fn{ii}); if isempty(typ), typ='fixed-point'; end
    elseif strncmp(fn{ii},'fb',2)|strncmpi(fn{ii},'fractionbits',5)
      fb=getfield(qdef,fn{ii}); if isempty(typ), typ='fixed-point'; end
    elseif strncmp(fn{ii},'Operation',2)
      op=getfield(qdef,fn{ii});

    elseif strncmpi(fn{ii},'Dither',2)
      dith=getfield(qdef,fn{ii});
    elseif strncmpi(fn{ii},'ToD',3)|strncmpi(fn{ii},'Type_of_Dither',6)
      tod=getfield(qdef,fn{ii});
    elseif strncmp(fn{ii},'L',1)|strncmpi(fn{ii},'NoB',3)|strncmpi(fn{ii},'Number_of_DitherBits',6)
      L=getfield(qdef,fn{ii});
    elseif strncmp(fn{ii},'A',1)|strncmpi(fn{ii},'Amplitude',9)
      A=getfield(qdef,fn{ii});
    elseif strncmpi(fn{ii},'Phase',2)
      ph=getfield(qdef,fn{ii});
    elseif strncmpi(fn{ii},'Var',3)|strncmpi(fn{ii},'Variance_of_Gaussian_Dither',8)
      gvar=getfield(qdef,fn{ii});
    elseif strncmpi(fn{ii},'RoF',3)|strcmpi(fn{ii},'Ratio_of_Frequencies')
      rof=getfield(qdef,fn{ii});
      if (length(rof) ~= 2) error('Size of vector [numerator, denominator] is wrong.');
      else rofn=rof(1); rofd=rof(2);
      end
    elseif strncmpi(fn{ii},'DFunction',4)|strncmpi(fn{ii},'DFunction_given_by_User',9)
      dfun=getfield(qdef,fn{ii});
    elseif strncmpi(fn{ii},'Sequenceperm',3)
      seqp=getfield(qdef,fn{ii});
      %     elseif strncmpi(fn{ii},'Other',5)
      %          error('It is not allowed to modify "Other" dither-properties.');

    else if ~(strncmpi(fn{ii},'Other',5)) error(['Unknown qdef field ''',fn{ii},'''']); end
    end
  end
elseif isnumeric(qdef)
  if rem(qdef,1)~=0, error('p is not an integer'), end
  Qout=qstruct(struct('p',qdef));
  return
end
%
if length(varargin)>=2
  if nargout==0, error('No output argument is given.'), end
  fn=varargin(1:2:end);
  for ii=1:length(fn)
    %eval([fn{ii},'=getfield(qdef,fn{ii});'])
    if strcmp(fn{ii},'p')|strcmp(fn{ii},'prec')|strcmpi(fn{ii},'precision')
      p=varargin{2*ii}; if isempty(typ), typ='floating-point'; end
    elseif strncmpi(fn{ii},'Type',2)
      typ=varargin{2*ii};
    elseif strncmpi(fn{ii},'Name',3)
      Name=varargin{2*ii};
    elseif strcmp(fn{ii},'lb')|strcmpi(fn{ii},'leadingbit')
      lb=varargin{2*ii}; if isempty(typ), typ='floating-point'; end
    elseif strcmp(fn{ii},'rdir')|strncmpi(fn{ii},'roundingdirection',11)
      rdir=varargin{2*ii};
    elseif strcmp(fn{ii},'treath')|strncmpi(fn{ii},'treathalf',6)
      treath=varargin{2*ii};
    elseif strncmpi(fn{ii},'coding',1)
      coding=varargin{2*ii};
    elseif strcmp(fn{ii},'ufl')|strcmpi(fn{ii},'underflow')
      ufl=varargin{2*ii}; if isempty(typ), typ='floating-point'; end
    elseif strcmpi(fn{ii},'Emin')|strncmpi(fn{ii},'Expminimum',6)
      Emin=varargin{2*ii}; if isempty(typ), typ='floating-point'; end
    elseif strcmpi(fn{ii},'Emax')|strncmpi(fn{ii},'Expmaximum',6)
      Emax=varargin{2*ii}; if isempty(typ), typ='floating-point'; end
    elseif strcmpi(fn{ii},'Ebias')|strncmpi(fn{ii},'Expbias',6)
      Ebias=varargin{2*ii}; if isempty(typ), typ='floating-point'; end
    elseif strcmp(fn{ii},'B')|strncmpi(fn{ii},'bits',4)
      B=varargin{2*ii}; if isempty(typ), typ='fixed-point'; end
    elseif strcmpi(fn{ii},'xnmin')|strncmpi(fn{ii},'minimum',3)
      xnmin=varargin{2*ii}; if isempty(typ), typ='fixed-point'; end
    elseif strcmpi(fn{ii},'xpmax')|strncmpi(fn{ii},'maximum',3)
      xpmax=varargin{2*ii}; if isempty(typ), typ='fixed-point'; end
    elseif strcmpi(fn{ii},'ovf')|strncmpi(fn{ii},'overflow',5)
      ovf=varargin{2*ii}; if isempty(typ), typ='fixed-point'; end
    elseif strncmpi(fn{ii},'fb',2)|strncmpi(fn{ii},'fractionbits',5)
      fb=varargin{2*ii}; if isempty(typ), typ='fixed-point'; end
      if ~isempty(xnmin)&~isempty(B)
        xnmin=-pow2((B-1)-fb); xpmax=pow2((B-1)-fb)-1;
        if strcmp(coding,'o')|strcmp(coding,'s'), xnmin=xnmin+1; end
      end
    elseif strncmpi(fn{ii},'Operation',2)
      op=varargin{2*ii};

    elseif strncmpi(fn{ii},'Dither',2)
      dith=varargin{2*ii};
    elseif strncmpi(fn{ii},'ToD',3)
      tod=varargin{2*ii};
    elseif strcmp(fn{ii},'L')|strncmpi(fn{ii},'NoB',3)|strncmpi(fn{ii},'Number_of_DitherBits',6)
      L=varargin{2*ii};
    elseif strcmp(fn{ii},'A')|strncmpi(fn{ii},'Amplitude',9)
      A=varargin{2*ii};
    elseif strncmpi(fn{ii},'Phase',2)
      ph=varargin{2*ii};
    elseif strncmpi(fn{ii},'Var',3)|strncmpi(fn{ii},'Variance_of_Gaussian_Dither',8)
      gvar=varargin{2*ii};
    elseif strncmpi(fn{ii},'RoF',3)|strcmpi(fn{ii},'Ratio_of_Frequencies_[num. denom.]')
      rof=varargin{2*ii};
      if (length(rof) ~= 2)
        error('Size of vector [numerator, denominator] is wrong.');
      else rofn=rof(1); rofd=rof(2);
      end
    elseif strncmpi(fn{ii},'DFunction',4)|strncmpi(fn{ii},'DFunction_given_by_User',9)
      dfun=varargin{2*ii};
    elseif strncmpi(fn{ii},'Sequenceperm',3)
      seqp=varargin{2*ii};
      %     elseif strncmpi(fn{ii},'Other',5)
      %         error('It is not allowed to modify "Other" dither-properties.');

    else if ~(strncmpi(fn{ii},'Other',5)) error(['Unknown qdef field ''',fn{ii},'''']); end
    end
  end
end
%
if isempty(typ)
  if ~isempty(B)|~isempty(xnmin), typ='fixed-point';
  elseif ~isempty(p)|~isempty(ufl)|~isempty(lb)|~isempty(Emin), typ='floating-point';
  else error('Cannot determine type of quantizer')
  end
end

if strncmp(typ,'float',2)
  if ~exist('p'), p=[]; end, if isempty(p), p=53; end
  if ~isempty(fb)|(fb>p), fb=p-1; end
  %
  if ~exist('lb'), lb=''; end, if isnan(lb), lb=''; end
  if isempty(lb), lb='h'; end %hidden leading bit
  if ~exist('rdir'), rdir=''; end, if isnan(rdir), rdir=''; end
  if isempty(rdir), rdir='r'; end %round
  if ~exist('treath'), treath=''; end, if isnan(treath), treath=''; end
  if isempty(treath), treath='e'; end %round to even
  if ~exist('coding'), coding=''; end, if isempty(coding), coding='s'; end %sign-magnitude
  if ~exist('ufl'), ufl=''; end, if isnan(ufl), ufl=''; end
  if isempty(ufl), ufl='g'; end %gradual underflow
  %
  Eminlim=round(log2(realmin)); Emaxlim=round(log2(realmax))-1;
  %For IEEE double precision, Emin=-1022, Emax=1023
  if ~exist('Emin'), Emin=[]; end, if isnan(Emin), Emin=[]; end
  if isempty(Emin), Emin=Eminlim; end
  if ~exist('Emax'), Emax=[]; end, if isnan(Emax), Emax=[]; end
  if isempty(Emax), Emax=Emaxlim; end
else %fixed-point
  if isempty(B), B=16; end
  if isnan(fb), fb=[]; end
  if isempty(fb), fb=0; end %decimal point at right
  if isnan(rdir), rdir=''; end
  if isempty(rdir), rdir='r'; end %rounding
  if isnan(treath), treath=''; end
  if isempty(treath), treath='e'; end %round to even
  if isnan(coding), coding=''; end
  if isempty(coding), coding='t'; end %two's complement
  if isnan(ovf), ovf=''; end
  if isempty(ovf), ovf='m'; end %modular
  if isnan(xnmin), xnmin=[]; end
  if strncmp(coding,'t',1), xnminl=-pow2(B-fb-1); else xnminl=-pow2(pow2(B-1)-1,-fb); end
  if isempty(xnmin), xnmin=xnminl; end
  if isnan(xpmax), xpmax=[]; end
  xpmaxl=pow2(pow2(B-1)-1,-fb);
  if isempty(xpmax), xpmax=xpmaxl; end
end


if ~strcmpi(dith,'on')
  dith='off';
  Q=[]; if isstruct(qdef), Q=qdef; end
  Q.Name=Name;
  Q.Type=typ;
  Q.Precision=p;
  Q.FractionBits=fb;
  Q.RoundingDir=rdir;
  Q.TreatHalf=treath;
  Q.Coding=coding;
  Q.Dither=dith;

else
  dith='on';
  if (exist('L','var'))
    if (L==0)
      error('Number of DitherBits is 0.');
    end
  end
  if (~exist('L','var'))
    L=3;     %% Default Number of DitherBits
  end
  if (~exist('tod','var'))
    error('Type of Dither is missing.');
  end
  if ~exist('seqp','var'), seqp='disabled';
  else if strncmpi(seqp,'enable',2)
      seqp='enabled';
    else if strncmpi(seqp,'disable',3)
        seqp='disabled';
      else error('Sequence-property is wrong.');
      end
    end
  end


  if strcmpi(tod,'u1')|strcmpi(tod,'u2')|strcmpi(tod,'u3')|strncmpi(tod,'uniform',2)
    tod=lower(tod);
    if (~strcmp(tod,'u1'))
      if (~strcmp(tod,'u2'))
        if (~strcmp(tod,'u3'))
          tod='u2';
        end
      end
    end

    if strncmpi(typ,'fixed',2)|strncmpi(typ,'floating',2)
      lsb_value = 1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,1); %lsb_value -> 1
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    else
      lsb_value = 1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,1); %lsb_value -> 1
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    end

    if (exist('A','var'))  warning('Amplitude is unnecessary by uniform dither.'); end
    if (exist('ph','var'))  warning('Phase is unnecessary by uniform dither.'); end
    if (exist('gvar','var'))  warning('Variance is unnecessary by uniform dither.'); end
    if (exist('rofn','var'))  warning('Ratio of frequencies (num.) is unnecessary by uniform dither.'); end
    if (exist('rofd','var'))  warning('Ratio of frequencies (denum.) is unnecessary by uniform dither.'); end

    Q=struct('Name',Name,'Type',typ,'Precision',p,'FractionBits',fb,'RoundingDir',rdir,...
      'TreatHalf',treath,'Coding',coding,'Dither',dith,'ToD',tod,'Number_of_DitherBits',L,'Sequenceperm',seqp,'Other',other);

  end
  if strncmpi(tod,'triangular',1)
    tod=lower(tod);
    if (~strcmp(tod,'t1'))
      if (~strcmp(tod,'t2'))
        if (~strcmp(tod,'t3'))
          tod='t2';
        end
      end
    end

    if strncmpi(typ,'fixed',2)|strncmpi(typ,'float',2)
      lsb_value = 1; %1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    else
      lsb_value = 1; %1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    end

    if (exist('A','var'))  warning('Amplitude is unnecessary by triangular dither.'); end
    if (exist('ph','var'))  warning('Phase is unnecessary by triangular dither.'); end
    if (exist('gvar','var'))  warning('Variance is unnecessary by triangular dither.'); end
    if (exist('rofn','var'))  warning('Ratio of frequencies (num.) is unnecessary by triangular dither.'); end
    if (exist('rofd','var'))  warning('Ratio of frequencies (denum.) is unnecessary by triangular dither.'); end

    Q=struct('Name',Name,'Type',typ,'Precision',p,'FractionBits',fb,'RoundingDir',rdir,...
      'TreatHalf',treath,'Coding',coding,'Dither',dith,'ToD',tod,'Number_of_DitherBits',L,'Sequenceperm',seqp,...
      'Other',other);
  end

  if strncmpi(tod,'Gaussian',2)
    tod='Gaussian';

    if ~exist('gvar','var') error('Variance of Gaussian Dither is missing.'); end
    if (gvar == 0) error('Variance of Gaussian Dither is 0.'); end

    if strncmpi(typ,'fixed',2)
      lsb_value = 1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    else
      lsb_value = 1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    end

    Q=struct('Name',Name,'Type',typ,'Precision',p,'FractionBits',fb,'RoundingDir',rdir,...
      'TreatHalf',treath,'Coding',coding,'Dither',dith,'ToD',tod,'Number_of_DitherBits',L,...
      'Variance_of_Gaussian_Dither',gvar,'Sequenceperm',seqp,'Other',other);

    if (exist('A','var'))  warning('Amplitude is unnecessary by Gaussian dither.'); end
    if (exist('ph','var'))  warning('Phase is unnecessary by Gaussian dither.'); end
    if (exist('rofn','var'))  warning('Ratio of frequencies (num.) is unnecessary by Gaussian dither.'); end
    if (exist('rofd','var'))  warning('Ratio of frequencies (denum.) is unnecessary by Gaussian dither.'); end


  end

  if strncmpi(tod,'Sinusoid',3)
    tod='sinusoid';

    if ~exist('A','var') error('Amplitude of Sinusoid Dither is missing.'); end
    if (A == 0) error('Amplitude of sinosoid dither is 0.'); end
    if ~exist('rofn','var') error('The numerator of frequency-ratio of Sinusoid Dither is missing.'); end
    if (rofn == 0) error('Ratio of frequencies (numerator) dither is 0.'); end
    if ~exist('rofd','var') error('The denumerator of frequency-ratio of Sinusoid Dither is missing.'); end
    if (rofd == 0) error('Ratio of frequencies (denumerator) is 0.'); end
    if ~exist('ph','var') error('Phase of Sinusoid Dither is missing.'); end

    if strncmpi(typ,'fixed',2)
      lsb_value = 1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    else
      lsb_value = 1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    end

    Q=struct('Name',Name,'Type',typ,'Precision',p,'FractionBits',fb,'RoundingDir',rdir,...
      'TreatHalf',treath,'Coding',coding,'Dither',dith,'ToD',tod,'Number_of_DitherBits',L,...
      'Amplitude_of_Sinusoid_Dither',A,'Ratio_of_Frequencies',[rofn,rofd],...
      'Phase',ph,'Sequenceperm',seqp,'Other',other);

    if (exist('gvar','var')) warning('Variance is unnecessary by sinusoidal dither.'); end

  end

  if strncmpi(tod,'user',2)
    tod='userfunction';
    if ~exist('dfun','var') error(['Dither function ''',dfun,''' is missing.']); end

    if strncmpi(typ,'fixed',2)
      lsb_value = 1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value,dfun);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    else
      lsb_value = 1/(2^(fb));
      [sdith,probab,hmtimes,sumhmtimes] = set_dither(tod,L,lsb_value,dfun);
      other.Sdith = sdith;
      other.Probab = probab;
      other.Hmtimes = hmtimes;
      other.SumHmtimes = sumhmtimes;
      other.NumberOfUsage = zeros(1,length(other.Hmtimes));
    end

    Q=struct('Name',Name,'Type',typ,'Precision',p,'FractionBits',fb,'RoundingDir',rdir,...
      'TreatHalf',treath,'Coding',coding,'Dither',dith,'ToD',tod, 'Dfunction',dfun,...
      'Number_of_DitherBits',L,'Sequenceperm',seqp,'Other',other);

    if (exist('A','var'))  warning('Amplitude is unnecessary by Gaussian dither.'); end
    if (exist('ph','var'))  warning('Phase is unnecessary by Gaussian dither.'); end
    if (exist('rofn','var'))  warning('Ratio of frequencies (num.) is unnecessary by Gaussian dither.'); end
    if (exist('rofd','var'))  warning('Ratio of frequencies (denum.) is unnecessary by Gaussian dither.'); end

  end
end

if strncmp(typ,'float',2)
  Q.LeadingBit=lb; Q.UnderFlow=ufl; Q.ExpMinimum=Emin; Q.ExpMaximum=Emax; Q.ExpBias=Ebias;
elseif strncmp(typ,'fix',2)
  Q.Bits=B; Q.Minimum=xnmin; Q.Maximum=xpmax; Q.OverFlow=ovf;
end
Q.Operation=op;

qstruct(Q,'test'); %test consistency

if length(varargin)==1
  [fn,qpropsugg]=qprops(varargin{1})
  fprintf([qpropsugg,'\n'])
  if nargout>0, Qout=sprintf([qpropsugg,'\n']); end
  return
end
%make list
if nargout==0, qstructget(Q), else Qout=Q; end
%
%End of qstruct
