function x=roundifft(X,Q,runmod)
%ROUNDIFFT Quantized IFFT algorithm
%
%   x=roundifft(X,Q)
%
%   Input arguments:
%     X = input sequence (2's power)
%     Q = definition of the quantizer (if not given, IEEE double)
%     runmod = run modifier, see roundfft
%   If X is qdata, then rounding is qdata-based
%        x=roundifft(X,runmod)
%
%   Output arguments:
%     x = vector, or structure for block-float 
%          fields for bf: data, bfe
%
%   Example:
%     roundifft(roundfft([1:8]))

%	Copyright (c) I. Kollar, 2006, all rights reserved
%	Last modified: 21-Jul-2008

if isa(X,'qdata'), error(nargchk(1,2,nargin)), end
if (isa(X,'qdata')&(nargin<2))|(nargin<3), runmod=[]; end
if isempty(runmod), runmod=struct('direction','inverse'); 
elseif isstruct(runmod), runmod.direction='inverse'; 
else error('runmod is not allowed')
end
if isa(X,'qdata'), x=roundfft(X,runmod);
else
  if nargin<2, Q='double'; end, if isempty(Q), Q='double'; end
  x=roundfft(X,Q,runmod);
end
%