function [wm,rm,zm]=tf2res(num,den,zm_in)
%TF2RES  Resonator-based filter implementation
%
%       the poles zm can be fixed at the input
%       num and den are polynomials of z^-1

%	Copyright (c) I. Kollar, 2008, all rights reserved
%	Last modified: 23-Aug-2008
%
% See also: resfiltq

%Literature:
%Gbor Pceli, "Resonator-based Digital Filters," IEEE Trans. on 
%Circuits and Systems, Vol. 36, No. 1, Jan. 1989, pp. 156-159.

N=max([length(den),length(num),3])-1;
if den(1)~=0, num=num/den(1); den=den/den(1); 
else error('den(1)= 0')
end
if num(1)~=0
  b0=num(1); 
  numl=num; denl=den;
  while length(numl)<length(denl), numl=[numl,0]; end
  while length(numl)>length(denl), denl=[denl,0]; end
  numm=numl-b0*denl;
else
  numm=num; b0=[];
end
if nargin<3, zm_in=[]; end, zm=zm_in;
if (length(zm)==1)&isnumeric(zm)&(rem(zm,1)==0)&(zm>1)
  %number of poles is given
  N=zm; zm=[];
elseif strncmpi(zm,'optimal',3)
  if length(num)>length(den)
    zm=[];
  else
    %find optimal zm positions
    numP=fliplr(den);
    if findstr(zm,'2'), zm=roots(den+numP);
    else zm=roots(den-numP);
    end
    wzm=angle(zm); ind=find(wzm<0); phzm=wzm; phzm(ind)=phzm(ind)+2*pi;
    [phzm,ind]=sort(phzm);
    zm=zm(ind); wzm=wzm(ind);
  end
else
  zm=zm(:);
end
if isempty(zm)
  zm=exp(j*[0:N-1]'/N*2*pi); wzm=angle(zm);
else %zm given
  if any(abs(abs(zm)-1)>10*eps), error('The zm''s are not on the unit circle'), end
  wzm=angle(zm); ind=find(wzm<0); phzm=wzm; phzm(ind)=phzm(ind)+2*pi;
  [phzm,ind]=sort(phzm); 
  if ~isequal(zm,zm(ind)), error('zm_in is not in order around the unit circle'), end
  %zm=zm(ind); wzm=wzm(ind); 
  if (length(zm)<length(den)-1)|(length(zm)<length(num)-1)|...
      (N<length(den)-1)|(N<length(num)-1)
    error('Cannot design filter with less resonators than order')
  end
  if N>length(zm)
    error(sprintf('Cannot realize resonator-based filter with %.0f poles',length(zm)))
  end
  N=length(zm);
end
wm=freqz(numm,den,wzm);
wm=[wm(:);b0];
%
if nargout>=2,
  rm=zeros(N,1);
  denN=den; while length(denN)<N+1, denN=[denN,0]; end
  %length(den)-1, N
  for m=0:N-1
    rm(m+1)=polyval(denN,zm(m+1)) / prod(1-zm([1:m,m+2:N])/zm(m+1));
    if (m==0)|(m==N/2), rm(m+1)=real(rm(m+1)); end
  end
  ind=find(abs(imag(rm))<100*eps); if ~isempty(ind), rm(ind)=real(rm(ind)); end
  %if any(imag(rm)), warning('rm is complex'), end
  %rm=real(rm); %sometimes it is complex
end
if rem(N,2)==0, 
  zm(N/2+1)=real(zm(N/2+1)); wm(N/2+1)=real(wm(N/2+1)); 
  if nargin>=2, rm(N/2+1)=real(rm(N/2+1)); end
end
